/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
/**
 *
 */
package com.sos.JSHelper.Options;

import java.io.UnsupportedEncodingException;

import com.sos.JSHelper.Exceptions.JobSchedulerException;

/**
 * @author KB
 *
 */
public class SOSOptionHexString extends SOSOptionFileString {

	/**
	 *
	 */
	private static final long	serialVersionUID	= 5459978964312384049L;
	@SuppressWarnings("unused")
	private final String	conClassName	= "SOSOptionHexString";

	/**
	 * @param pPobjParent
	 * @param pPstrKey
	 * @param pPstrDescription
	 * @param pPstrValue
	 * @param pPstrDefaultValue
	 * @param pPflgIsMandatory
	 */
	public SOSOptionHexString(final JSOptionsClass pPobjParent, final String pPstrKey, final String pPstrDescription, final String pPstrValue, final String pPstrDefaultValue,
			final boolean pPflgIsMandatory) {
		super(pPobjParent, pPstrKey, pPstrDescription, pPstrValue, pPstrDefaultValue, pPflgIsMandatory);
	}

	@Override
	public String Value() {

		/**
		 * TODO reconvert XML Entities. Create a separate Methode: reconvertXMLEntities()
		 * this Mthod should be implemented in one of the base classes
		 */
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::Value";

		String strV = strValue;
		if (isNotEmpty(strV) && isHex(strV)) {
			try {
				strV = new String(fromHexString(strV), "US-ASCII");
			}
			catch (UnsupportedEncodingException e) {
				e.printStackTrace(System.err);
			}
			catch (IllegalArgumentException e) {
				e.printStackTrace(System.err);
			}
		}
		return strV;
	} // private String Value

	/**
	 * Unescapes a string containing XML entity escapes to a string containing the actual Unicode characters corresponding to the escapes.

	 *  Supports only the five basic XML entities (gt, lt, quot, amp, apos).
	 * @return
	 */
	public String unescapeXML() {

		@SuppressWarnings("unused")
		final String	conMethodName	= conClassName + "::unescapeXML";

		String newValue = this.Value();
		newValue = newValue.replaceAll("&quot;", "\"");
		newValue = newValue.replaceAll("&lt;", "<");
		newValue = newValue.replaceAll("&gt;", ">");
		newValue = newValue.replaceAll("&amp;", "&");
		newValue = newValue.replaceAll("&apos;", "'");

		return newValue;

	} // private String unescapeXML

	public byte[] fromHexString() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::fromHexString";

		return this.fromHexString(strValue);

	} // private byte[] fromHexString

	public byte[] fromHexString(final String s) throws IllegalArgumentException {

		int stringLength = s.length();
		if ((stringLength & 0x1) != 0) {
			throw new JobSchedulerException(String.format("fromHexString '%1$s' requires an even number of hex characters", s));
		}
		byte[] b = new byte[stringLength / 2];

		for (int i = 0, j = 0; i < stringLength; i += 2, j++) {
			int high = charToNibble(s.charAt(i));
			int low = charToNibble(s.charAt(i + 1));
			b[j] = (byte) (high << 4 | low);
		}
		return b;
	}

	private int charToNibble(final char c) {

		if ('0' <= c && c <= '9') {
			return c - '0';
		}
		else
			if ('a' <= c && c <= 'f') {
				return c - 'a' + 0xa;
			}
			else
				if ('A' <= c && c <= 'F') {
					return c - 'A' + 0xa;
				}
				else {
					throw new JobSchedulerException("Invalid hex character: " + c);
				}
	}

	public boolean isHex() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::isHex";

		boolean flgRet = this.isHex(strValue);

		return flgRet;
	} // private boolean isHex

	public final boolean isHex(final String pstrHexString) {
		boolean flgRet = false;

		if (isNotEmpty(pstrHexString) ) {
			flgRet = true;
			for (int i = 0; i < pstrHexString.length(); i++) {
				if (isHexStringChar(pstrHexString.charAt(i)) == false) {
					flgRet = false;
					break;
				}
			}
		}
		return flgRet;
	}

	public final boolean isHexStringChar(final char c) {
		return Character.isDigit(c) || Character.isWhitespace(c) || "0123456789abcdefABCDEF".indexOf(c) >= 0;
	}

}
