/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jobnet.creator;

import java.util.Iterator;

import org.apache.log4j.Logger;

import com.sos.graphviz.enums.FileType;
import com.sos.graphviz.Graph;
import com.sos.graphviz.GraphIO;
import com.sos.jobnet.classes.JobNetException;
import com.sos.jobnet.creator.jobs.JobNetCreatorOptions;

public class JobNetToGraphiz {

	private final static Logger logger = Logger.getLogger(JobNetToGraphiz.class);

	private final String graphType;
	private final String graphDir;
	
	private Graph graph = new Graph();

	public JobNetToGraphiz(JobNetCreatorOptions options) {
		this.graphType = options.GraphType.Value();
		this.graphDir = options.GraphDir.Value();
	}

	public void createGraph(JobNetBootstrapOrder bootStrapOrder) {
//		for(JobNetOrder order : bootStrapOrder.getJobNet().values()) {
//		}
		addNodePairs(bootStrapOrder);
		try {
			String fileName = graphDir + "/" + bootStrapOrder.getJobNetOrderName() + "." + graphType;
			logger.info("writing graph to " + fileName);
			logger.info(graph.getSource());
			GraphIO graphIO = new GraphIO(graph);
			graphIO.writeGraphToFile(FileType.fromText(graphType), fileName );
		} catch (Exception e) {
			logger.error(e.getMessage());
			throw new JobNetException(e.getMessage(),e);
		}
		
	}
	
	private void addNodePairs(JobNetOrder order) {
		Iterator<JobNetOrder> it = order.getSuccessors().iterator();
		while(it.hasNext()) {
			JobNetOrder successor = it.next();
			logger.info(order.getId() + " -> " + successor.getId());
			graph.newEdge( graph.newNode(order.getJobNetOrderName()), graph.newNode(successor.getJobNetOrderName()) );
			addNodePairs(successor);
		}
	}

}
