/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jobnet.jobs;

//import JobSchedulerJobAdapter;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.Priority;

import sos.scheduler.job.JobSchedulerJobAdapter;
import sos.spooler.Order;
import sos.spooler.Supervisor_client;
import sos.spooler.Variable_set;

import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.JSHelper.Options.SOSOptionString;
import com.sos.JSHelper.Options.SOSOptionUUID;
import com.sos.jitl.eventing.JSEventsClient;
import com.sos.jitl.eventing.JSEventsClientOptions;
import com.sos.jobnet.classes.JobNetConstants;
import com.sos.jobnet.classes.JobNetOrderParams;
import com.sos.jobnet.classes.JobNetProtocol;
import com.sos.jobnet.classes.StatusProtocol;
import com.sos.jobnet.creator.JobNetIdCalculator;
import com.sos.jobnet.creator.jobs.JobNetIdCalculatorOptions;
import com.sos.jobnet.dispatcher.JobNetDispatcher;
import com.sos.jobnet.interfaces.IJobNetCallback;
import com.sos.jobnet.interfaces.IPlanItem;
import com.sos.jobnet.classes.Status.enuStatus;
import com.sos.jobnet.options.JobNetOptions;
import com.sos.jobnet.waiter.JobNetWaiter;
import com.sos.scheduler.model.SchedulerObjectFactory;
import com.sos.scheduler.model.answers.Answer;
import com.sos.scheduler.model.answers.ERROR;
import com.sos.scheduler.model.commands.JSCmdAddOrder;
import com.sos.scheduler.model.objects.JSObjOrder;
import com.sos.scheduler.model.objects.Param;
import com.sos.scheduler.model.objects.Params;
import com.sos.scheduler.model.objects.Spooler;

/**
 * \class JobNetWaitAndDispatch 
 * 
 * \brief JobNetWaitAndDispatch - 
 * 
 * \details
 *
 *
 * \code
 *   .... code goes here ...
 * \endcode
 *
 * <p style="text-align:center">
 * <br />---------------------------------------------------------------------------
 * <br /> APL/Software GmbH - Berlin
 * <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
 * <br />---------------------------------------------------------------------------
 * </p>
 * \author KB
 * \version $Id$
 * \see reference
 *
 * Created on 28.02.2012 13:10:54
 */

/**
 * @author KB
 * 
 */
public class JobNetWaitAndDispatch extends JobSchedulerJobAdapter implements
		IJobNetCallback {

	private final String conClassName = "JobNetWaitAndDispatch";
	@SuppressWarnings("unused")
	private static final String conSVNVersion = "$Id$";

	private static final Logger logger = Logger.getLogger(JobNetWaitAndDispatch.class);
	private JobNetProtocol jobnetProtocol = null;
	private StatusProtocol statusProtocol = null;
	private boolean flgRunnerIsSkipped = false;
	private boolean flgWaiterOK = true;
	private boolean flgDispatcherDone = false;
//	private Logger jobnetProtocol = null;

	public JobNetWaitAndDispatch() {
		//
	}

	public boolean spooler_process_before() {
		final String conMethodName = conClassName + "::spooler_process_before";
		flgRunnerIsSkipped = false;
		flgWaiterOK = true;
		flgDispatcherDone = false;
		try {
			initialize();
			logger.info(conMethodName);
			
			JobNetWaiter jobNetWaiter = new JobNetWaiter();
			jobNetWaiter.setIJobNetCallback(this);
			JobNetOptions objO = jobNetWaiter.Options();
			jobNetWaiter.setOptionsUsingHashMap(getAllParametersAsProperties());
			objO.scheduler_id.Value(this.spooler.id());
			//TODO getJobCHainName
//			String jobChain = "/"+spooler_task.order().job_chain().name();
			String orderId = spooler_task.order().id();
			logger.info(String.format("Checking order %1s in job chain %2s ",orderId,objO.jobnet.Value()));

			initializeProtocol(orderId, objO.jobnet,objO.uuid_jobnet_identifier);
			jobnetProtocol.log(Level.INFO,String.format("%1s is started.",orderId));

//			objO.jobnet.Value(jobChain);
			objO.node.Value(orderId); 			
			
			setCheckMandatory(objO);
			objO.CheckMandatory();
			jobNetWaiter.doInit();
			flgWaiterOK = jobNetWaiter.run();
		} catch (Exception e) {
			flgWaiterOK = false;
			raiseException(e);
		}
		log(Level.INFO,"Waiter finished.");
		
		
		// run the dispatcher immediately and set the next state if runner is skipped
		// otherwise the order will be suspended if spooler_process_before ends with false 
		if (flgRunnerIsSkipped  && flgWaiterOK) {
			if (jobnetProtocol != null) jobnetProtocol.info("Processing of the order is skipped, because the runtime of the order does not allow the execution.");
			do_task_after();
			String nextState = getLastStateOfChain(spooler_task.order().state());
			log(Level.INFO,"Order is set to state " + nextState);
			spooler_task.order().set_state(nextState);
		}
		
		boolean result = (flgRunnerIsSkipped == false && flgWaiterOK);
		log(Level.INFO,"Waiter endet with: " + result);
		return result;
	}
	
	public void spooler_task_after() {

		// is already done if runner was skipped (see spooler_process_before)
		if (flgWaiterOK && !flgDispatcherDone) {
			try {
				do_task_after();
			} catch (Exception e) {
				raiseException(e);
			}
		}
		log(Level.INFO,"Order is finished.");

	}
	
	private String getLastStateOfChain(String state) {
		String currentState = spooler_task.order().job_chain().node(state).next_state();
		String nextState = spooler_task.order().job_chain().node(currentState).next_state();
		while(nextState != null) {
			currentState = nextState;
			nextState = spooler_task.order().job_chain().node(currentState).next_state();
		};
		return currentState;
	}

	private void do_task_after() {
		final String conMethodName = conClassName + "::spooler_task_after";
		try {
			initialize();
			logger.info(conMethodName);
			JobNetDispatcher jobNetDispatcher = new JobNetDispatcher();
			jobNetDispatcher.setIJobNetCallback(this);
			JobNetOptions dispatcherOptions = jobNetDispatcher.Options();
			String orderId = spooler_task.order().id();
			jobNetDispatcher.setOptionsUsingHashMap(getAllParametersAsProperties());

			initializeProtocol(orderId, dispatcherOptions.jobnet,dispatcherOptions.uuid_jobnet_identifier);
			logger.info(String.format("Checking order %1s in job chain %2s ",orderId,dispatcherOptions.jobnet.Value()));

			dispatcherOptions.scheduler_id.Value(this.spooler.id());
			dispatcherOptions.node.Value(orderId);
			setCheckMandatory(dispatcherOptions);
			jobNetDispatcher.Options().CheckMandatory();
			jobNetDispatcher.doInit();
			jobNetDispatcher.run();
			
			// in case of successful execution of the order an event is thrown
			String scope = (spooler_task.exit_code() == 0) ? "success" : "error";
			String eventId = getEventId(scope);
			if(!eventId.isEmpty()) {
				log(Level.INFO,"Try to throw event " + eventId);
				submitEvent(eventId);
			} else {
				log(Level.INFO,"No event to throw.");
			}
		
			// put an end notification in the protocol
			log(Level.INFO,String.format("End: %1s.",orderId));
			flgDispatcherDone = true;

		} catch (Exception e) {
			raiseException(e);
		}
	}
	
	/**
	 * The event will be unique by appending the id of the jobnet.
	 * 
	 * @param uuId
	 * @return
	 */
	private String getEventId(String scope) {
		String eventName = (scope.equals("success")) 
				? spooler_task.order().params().value(JobNetOrderParams.event_on_success.name())
				: spooler_task.order().params().value(JobNetOrderParams.event_on_error.name());
				
		return eventName;
//		String result = "";
//		if (!eventName.isEmpty()) {
//			String eventKeyName = spooler_task.order().params().value(JobNetOrderParams.jobnet_id_parameter_name.name());
//			String eventKey = spooler_task.order().params().value(eventKeyName);
//			result = eventName + "." + eventKey;
//		}
//		return result;
	}

	private void initialize() {
		initializeLog4jAppenderClass();
	}

	private void initializeProtocol(String orderId, SOSOptionString jobnet, SOSOptionUUID uuid) {
		jobnetProtocol = new JobNetProtocol(orderId, jobnet, uuid);
		statusProtocol = new StatusProtocol(orderId, jobnet, uuid);
	}

	private void raiseException(Exception e) {
		e.printStackTrace(System.err);
		throw new JobSchedulerException(e.getMessage(), e);
	}

	// TODO move to optionsClass !!!!!!!!!!!!!!!!!!!!!!!
	private void setCheckMandatory(JobNetOptions objOptions) {
		//Das fliegt hier raus und kommt in die Options.
		objOptions.uuid_jobnet_identifier.isMandatory(true);
		objOptions.hibernate_connection_config_file.isMandatory(true);
		objOptions.scheduler_id.isMandatory(true);
		objOptions.jobnet.isMandatory(true);
		objOptions.node.isMandatory(true);
 	}

	@Override
	public int getExitCode() {
		return spooler_task.exit_code();
	}

	@Override
	public void startNode(IPlanItem planItem) {
		
		
		logger.info(String.format("Starting Plan.Node: %1s.%2s",planItem.getPlanId(),planItem.getNodeId()));
		logger.debug(String.format("  Node: %1s",planItem.getJobnetNodeItem().getNode()));
		logger.debug(String.format("  in Net: %1s",planItem.getJobnetNodeItem().getJobnet()));
		logger.debug(String.format("  XML: %1s",planItem.getOrderXml()));
		
		SchedulerObjectFactory objSchedulerObjectFactory = null;
		objSchedulerObjectFactory = new SchedulerObjectFactory(	spooler.hostname(), spooler.tcp_port());
		objSchedulerObjectFactory.initMarshaller(Spooler.class);
		JSCmdAddOrder objAddOrder = objSchedulerObjectFactory.createAddOrder();

		JSObjOrder objOrder = new JSObjOrder(objSchedulerObjectFactory);
		
		objOrder.getOrderFromXMLString(planItem.getOrderXml());

		objAddOrder.setId(planItem.getJobnetNodeItem().getNode());
		objAddOrder.setJobChain(planItem.getJobnetNodeItem().getJobnet());
		// objAddOrder.setParams(objOrder.getParams());
		
		objAddOrder.setReplace("yes");
		// spooler_log.info("--------> " + objOrder.getState());
		objAddOrder.setState(objOrder.getState());
		objAddOrder.setTitle(objOrder.getTitle());
		objAddOrder.setAt("now");
		
 		Params objParams = objOrder.getParams();

 		for(Object o : objParams.getParamOrCopyParamsOrInclude()) {
 			if(o instanceof Param) {
	 			Param p = (Param)o;
	 			logger.info("Param from order: " + p.getName() + "=" + p.getValue());
 	 		}
 		}
 		
 		setParam(objParams,JobNetConstants.UUID_PARAMETER_NAME,planItem.getUuid());
 		setParam(objParams,JobNetConstants.JOB_NET_PARAMETER_NAME,planItem.getJobnetNodeItem().getJobnet());
 		
 		Order currentOrder = spooler_task.order();
 		if(currentOrder == null)
 			throw new JobSchedulerException("The order object is not set for the current order.");
 		
 		/**
 		 * If the parameter "ResultParameterName" is set a JobNetId will calculated and set to this field.
 		 */
        String targetName = currentOrder.params().value("ResultParameterName");
        if(!targetName.isEmpty()) {
    		String targetValue = calculateJobNetId();
	        spooler_log.debug("Set parameter .........: " + targetName + "=" + targetValue);
	 		setParam(objParams,targetName,targetValue);
        }

 		mergeParams(currentOrder.params(), objParams);
 		objAddOrder.setParams(objParams);		

		try{
			log(Level.INFO,String.format("Dispatching order: %1s.",objAddOrder.getId()));
			log(Level.DEBUG,objAddOrder.toXMLString());
    		objAddOrder.run();
		} catch (Exception e) {
			String msg = "Error starting order " + objAddOrder.getId();
			log(Level.ERROR,msg);
			logger.error(msg,e);
			throw new JobSchedulerException(msg,e);
		}
		
		log(Level.DEBUG,"Command submitted - waiting for answer ...");
		Answer objA = objOrder.getAnswer();
		ERROR objE = objA.getERROR();
		log(Level.DEBUG,"JS answer received.");
		if (objE != null) {
			log(Level.ERROR,"The answe contains an error - order not started. Errortext from JS: " + objE.getText());
		}

		try {
			objSchedulerObjectFactory.getSocket().doClose();
		} catch (IOException e) {
			String msg = "Error closing JobScheduler socket.";
			log(Level.ERROR,msg);
			throw new JobSchedulerException(msg,e);
		}

	}
	
	/**
	 * Calculates a jobnet id for an order
	 */
	private String calculateJobNetId() {
		String result = "";
		try {
			JobNetIdCalculatorOptions calcOptions = new JobNetIdCalculatorOptions();
			calcOptions.setAllOptions(getAllParametersAsProperties());
			JobNetIdCalculator calculator = new JobNetIdCalculator(calcOptions,isOrderJob());
			result = calculator.execute();
		} catch (Exception e) {
			String msg = "Error calculating JobNetId";
			log(Level.ERROR,msg);
			logger.error(msg,e);
			throw new JobSchedulerException(msg,e);
		}
		return result;
	}
	
	/**
	 * Puts the parameter from the current (running) order to the order to start. Placeholders like
	 * %varname% will be replaced with the real value of the parameter.
	 * Only params that no JobNetOrderParams itself (see class JobNetOrderParams) are merged.
	 * 
	 * @param fromVariables
	 * @param toParams
	 */
	private void mergeParams(Variable_set fromVariables, Params toParams) {
		HashMap<String,String> workParamsReplaced = getSchedulerParameterAsProperties(fromVariables);
		HashMap<String,String> fromParamsReplaced = removeAdditionalParameters(workParamsReplaced, fromVariables);
 		for(String key : fromParamsReplaced.keySet()) {
// 			if (key.equals("P_LOADID")) {
			if (!hasKey(toParams,key) && JobNetOrderParams.isPublicParameter(key)) {
 				setParam(toParams,key,fromParamsReplaced.get(key));
 	 			logger.info("variable merged from current order: " + key + "=" + fromParamsReplaced.get(key));
 			}
 		}
	}
	
	private HashMap<String,String> removeAdditionalParameters(HashMap<String,String> removeFrom, Variable_set original) {
		HashMap<String,String> result = new HashMap<String,String>();
		String[] names = original.names().split(";");
		List<String> originalList = Arrays.asList(names);
		for(String key : removeFrom.keySet()) {
			if(originalList.contains(key))
				result.put(key, removeFrom.get(key));
		}
		return result;
	}
	
	private static boolean hasKey(Params paramsToCheck, String key) {
		boolean hasKey = false;
		Iterator<Object> it = paramsToCheck.getParamOrCopyParamsOrInclude().iterator();
		while(it.hasNext()) {
			Object o = it.next();
			if(o instanceof Param) {
				Param p = (Param)o;
				if(p.getName().equals(key))
					hasKey = true;
			}
		}
		return hasKey;
	}
	
	private void setParam(Params toList, String name, String value) {
 		Param param = new Param();
		param.setValue(value);
		param.setName(name);
		toList.getParamOrCopyParamsOrInclude().add(param);
	}

	@Override
	public void skipRunner() {
		flgRunnerIsSkipped = true;
	}
	
	public void submitEvent(String eventId) {
		
		JSEventsClient eventClient = new JSEventsClient();
		JSEventsClientOptions eventOptions = eventClient.Options();

		try {
			logger.info("Try to throw event " + eventId);
			JobNetWaitAndDispatch.setSupervisorOptions(eventOptions, spooler);
			eventClient.setJSJobUtilites(this);
			eventOptions.CurrentNodeName(this.getCurrentNodeName());
			eventOptions.setAllOptions(getSchedulerParameterAsProperties(getJobOrOrderParameters()));
			eventOptions.scheduler_event_class.Value(this.getClass().getSimpleName());
			eventOptions.scheduler_event_id.Value(eventId.toLowerCase());
			eventOptions.CheckMandatory();
			eventClient.Execute();
			logger.info("Event succesfully thrown.");
		} catch (Exception e) {
			String msg = "Error creating event " + eventId; 
			logger.error(msg,e);
			throw new JobSchedulerException(msg,e);
		}

	}
	
	//TODO this method should be placed in JSEventsClient
	public static void setSupervisorOptions(JSEventsClientOptions options, sos.spooler.Spooler scheduler) {
		if (options.scheduler_event_handler_host.isDirty() == false) {  // no definition given ...
			try {
				Supervisor_client supervisor = scheduler.supervisor_client();
				options.scheduler_event_handler_host.Value(supervisor.hostname());
				options.scheduler_event_handler_port.value(supervisor.tcp_port());
			}
			catch (Exception e) { // there is no supervisor
				options.scheduler_event_handler_host.Value(scheduler.hostname());
				options.scheduler_event_handler_port.value(scheduler.tcp_port());
			}
		}
	}

	@Override
	public void setStateText(String stateText) {
//		if(stateText.length() > 96)
//			stateText = stateText.substring(0,96) + "...";		// maximum of state text in JobScheduler DB is 100
		this.spooler_task.order().set_state_text(stateText);
	}

	@Override
	public void setCC(int arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void log(Level level, String stateText, Throwable error) {
		if (jobnetProtocol != null)	jobnetProtocol.log(level,stateText, error);
		logger.log(level,stateText, error);
	}

	@Override
	public void log(Level level, String stateText) {
		if (jobnetProtocol != null)	jobnetProtocol.log(level,stateText);
	}

	@Override
	public void logStatus(IPlanItem record, enuStatus oldStatus) {
		enuStatus status = enuStatus.fromInt(record.getStatusNode());
		String msg = "nodestatus of id " + record.getNodeId() +  " is set from " + oldStatus.name() + " to " + status.name();
		if (jobnetProtocol != null)	jobnetProtocol.log(Level.INFO,msg);
		if (statusProtocol != null)	statusProtocol.log(Level.INFO,msg);
		logger.info(msg);
	}

	@Override
	public void logContextStatus(String context, enuStatus fromStatus, enuStatus toStatus) {
		String msg = context + " changed from  " + fromStatus.name() +  " to " + toStatus.name();
		if (jobnetProtocol != null)	jobnetProtocol.log(Level.INFO,msg);
		if (statusProtocol != null)	statusProtocol.log(Level.INFO,msg);
		logger.info(msg);
	}
		
}
