/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jobnet.options;
import java.util.HashMap;

import org.apache.log4j.Logger;

import com.sos.JSHelper.Annotations.JSOptionClass;
import com.sos.JSHelper.Annotations.JSOptionDefinition;
import com.sos.JSHelper.Exceptions.JSExceptionMandatoryOptionMissing;
import com.sos.JSHelper.Options.JSOptionsClass;
import com.sos.JSHelper.Options.SOSOptionBoolean;
import com.sos.JSHelper.Options.SOSOptionDBDriver;
import com.sos.JSHelper.Options.SOSOptionInFileName;
import com.sos.JSHelper.Options.SOSOptionInteger;
import com.sos.JSHelper.Options.SOSOptionJdbcUrl;
import com.sos.JSHelper.Options.SOSOptionPassword;
import com.sos.JSHelper.Options.SOSOptionSchedulerID;
import com.sos.JSHelper.Options.SOSOptionScript;
import com.sos.JSHelper.Options.SOSOptionString;
import com.sos.JSHelper.Options.SOSOptionTime;
import com.sos.JSHelper.Options.SOSOptionUUID;
import com.sos.hibernate.interfaces.IHibernateOptions;

/**
 * \class 		JobNetOptionsSuperClass - title
 *
 * \brief 
 * An Options-Super-Class with all Options. This Class will be extended by the "real" Options-class (\see JobNetOptions.
 * The "real" Option class will hold all the things, which are normaly overwritten at a new generation
 * of the super-class.
 *
 *
 *
 * see \see C:\Dokumente und Einstellungen\Uwe Risse\Lokale Einstellungen\Temp\scheduler_editor-3285541815265253193.html for (more) details.
 * 
 * \verbatim ;
 * mechanicaly created by C:\Dokumente und Einstellungen\Uwe Risse\Eigene Dateien\sos-berlin.com\jobscheduler.1.3.9\scheduler_139\config\JOETemplates\java\xsl\JSJobDoc2JSOptionSuperClass.xsl from http://www.sos-berlin.com at 20120224145540 
 * \endverbatim
 * \section OptionsTable Tabelle der vorhandenen Optionen
 * 
 * Tabelle mit allen Optionen
 * 
 * MethodName
 * Title
 * Setting
 * Description
 * IsMandatory
 * DataType
 * InitialValue
 * TestValue
 * 
 * 
 *
 * \section TestData Eine Hilfe zum Erzeugen einer HashMap mit Testdaten
 *
 * Die folgenden Methode kann verwendet werden, um fr einen Test eine HashMap
 * mit sinnvollen Werten fr die einzelnen Optionen zu erzeugen.
 *
 * \verbatim
 private HashMap <String, String> SetJobSchedulerSSHJobOptions (HashMap <String, String> pobjHM) {
	pobjHM.put ("		JobNetOptionsSuperClass.auth_file", "test");  // This parameter specifies the path and name of a user's pr
		return pobjHM;
  }  //  private void SetJobSchedulerSSHJobOptions (HashMap <String, String> pobjHM)
 * \endverbatim
 */
@JSOptionClass(name = "JobNetOptionsSuperClass", description = "JobNetOptionsSuperClass")
public class JobNetOptionsSuperClass extends JSOptionsClass implements IHibernateOptions{
	/**
	 * 
	 */
	private static final long	serialVersionUID	= 2628303960726088794L;
	private final String	conClassName	= "JobNetOptionsSuperClass";
	@SuppressWarnings("unused")
	private static Logger	logger			= Logger.getLogger(JobNetOptionsSuperClass.class);

/**
 * \var check_interval : 
 * 
 *
 */
    @JSOptionDefinition(name = "check_interval", 
    description = "", 
    key = "check_interval", 
    type = "SOSOptionTime", 
    mandatory = false)
    
    public SOSOptionTime check_interval = new SOSOptionTime(this, conClassName + ".check_interval", // HashMap-Key
                                                                "", // Titel
                                                                "00:00:30", // InitValue
                                                                "00:00:30", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getcheck_interval : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionTime  getcheck_interval() {
        return check_interval;
    }

/**
 * \brief setcheck_interval : 
 * 
 * \details
 * 
 *
 * @param check_interval : 
 */
    public void setcheck_interval (SOSOptionTime p_check_interval) { 
        this.check_interval = p_check_interval;
    }

                        

/**
 * \var delay_start_dispatcher : 
 * 
 *
 */
    @JSOptionDefinition(name = "delay_start_dispatcher", 
    description = "", 
    key = "delay_start_dispatcher", 
    type = "SOSOptionTime", 
    mandatory = false)
    
    public SOSOptionTime delay_start_dispatcher = new SOSOptionTime(this, conClassName + ".delay_start_dispatcher", // HashMap-Key
                                                                "", // Titel
                                                                "00:00:03", // InitValue
                                                                "00:00:03", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getdelay_start_dispatcher : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionTime  getdelay_start_dispatcher() {
        return delay_start_dispatcher;
    }

/**
 * \brief setdelay_start_dispatcher : 
 * 
 * \details
 * 
 *
 * @param delay_start_dispatcher : 
 */
    public void setdelay_start_dispatcher (SOSOptionTime p_delay_start_dispatcher) { 
        this.delay_start_dispatcher = p_delay_start_dispatcher;
    }

                        

/**
 * \var hibernate_connection_autocommit : 
 * 
 *
 */
    @JSOptionDefinition(name = "hibernate_connection_autocommit", 
    description = "", 
    key = "hibernate_connection_autocommit", 
    type = "SOSOptionBoolean", 
    mandatory = false)
    
    public SOSOptionBoolean hibernate_connection_autocommit = new SOSOptionBoolean(this, conClassName + ".hibernate_connection_autocommit", // HashMap-Key
                                                                "", // Titel
                                                                "false", // InitValue
                                                                "false", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief gethibernate_connection_autocommit : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionBoolean  gethibernate_connection_autocommit() {
        return hibernate_connection_autocommit;
    }

/**
 * \brief sethibernate_connection_autocommit : 
 * 
 * \details
 * 
 *
 * @param hibernate_connection_autocommit : 
 */
    public void sethibernate_connection_autocommit (SOSOptionBoolean p_hibernate_connection_autocommit) { 
        this.hibernate_connection_autocommit = p_hibernate_connection_autocommit;
    }

                        

/**
 * \var hibernate_connection_config_file : 
 * Hibernate configuration file of the database connection
 *
 */
    @JSOptionDefinition(name = "hibernate_connection_config_file", 
    description = "", 
    key = "hibernate_connection_config_file", 
    type = "SOSOptionInFileName", 
    mandatory = false)
    
    public SOSOptionInFileName hibernate_connection_config_file = new SOSOptionInFileName(this, conClassName + ".hibernate_connection_config_file", // HashMap-Key
                                                                "", // Titel
                                                                "hibernate.cfg.xml", // InitValue
                                                                "hibernate.cfg.xml", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief gethibernate_connection_config_file : 
 * 
 * \details
 * Hibernate configuration file of the database connection
 *
 * \return 
 *
 */
    public SOSOptionInFileName  gethibernate_connection_config_file() {
        return hibernate_connection_config_file;
    }

/**
 * \brief sethibernate_connection_config_file : 
 * 
 * \details
 * Hibernate configuration file of the database connection
 *
 * @param hibernate_connection_config_file : 
 */
    public void sethibernate_connection_config_file (SOSOptionInFileName p_hibernate_connection_config_file) { 
        this.hibernate_connection_config_file = p_hibernate_connection_config_file;
    }

                        

/**
 * \var hibernate_connection_driver_class : 
 * Class of JBDC driver of the database connection
 *
 */
    @JSOptionDefinition(name = "hibernate_connection_driver_class", 
    description = "", 
    key = "hibernate_connection_driver_class", 
    type = "SOSOptionDBDriver", 
    mandatory = false)
    
    public SOSOptionDBDriver hibernate_connection_driver_class = new SOSOptionDBDriver(this, conClassName + ".hibernate_connection_driver_class", // HashMap-Key
                                                                "", // Titel
                                                                "oracle.jdbc.driver.OracleDriver", // InitValue
                                                                "oracle.jdbc.driver.OracleDriver", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief gethibernate_connection_driver_class : 
 * 
 * \details
 * Class of JBDC driver of the database connection
 *
 * \return 
 *
 */
    public SOSOptionDBDriver  gethibernate_connection_driver_class() {
        return hibernate_connection_driver_class;
    }

/**
 * \brief sethibernate_connection_driver_class : 
 * 
 * \details
 * Class of JBDC driver of the database connection
 *
 * @param hibernate_connection_driver_class : 
 */
    public void sethibernate_connection_driver_class (SOSOptionDBDriver p_hibernate_connection_driver_class) { 
        this.hibernate_connection_driver_class = p_hibernate_connection_driver_class;
    }

                        

/**
 * \var hibernate_connection_password : 
 * Password of the database connection
 *
 */
    @JSOptionDefinition(name = "hibernate_connection_password", 
    description = "", 
    key = "hibernate_connection_password", 
    type = "SOSOptionPassword", 
    mandatory = false)
    
    public SOSOptionPassword hibernate_connection_password = new SOSOptionPassword(this, conClassName + ".hibernate_connection_password", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief gethibernate_connection_password : 
 * 
 * \details
 * Password of the database connection
 *
 * \return 
 *
 */
    public SOSOptionPassword  gethibernate_connection_password() {
        return hibernate_connection_password;
    }

/**
 * \brief sethibernate_connection_password : 
 * 
 * \details
 * Password of the database connection
 *
 * @param hibernate_connection_password : 
 */
    public void sethibernate_connection_password (SOSOptionPassword p_hibernate_connection_password) { 
        this.hibernate_connection_password = p_hibernate_connection_password;
    }

                        

/**
 * \var hibernate_connection_url : 
 * JDBC URL of the database connection
 *
 */
    @JSOptionDefinition(name = "hibernate_connection_url", 
    description = "", 
    key = "hibernate_connection_url", 
    type = "SOSOptionJdbcUrl", 
    mandatory = false)
    
    public SOSOptionJdbcUrl hibernate_connection_url = new SOSOptionJdbcUrl(this, conClassName + ".hibernate_connection_url", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief gethibernate_connection_url : 
 * 
 * \details
 * JDBC URL of the database connection
 *
 * \return 
 *
 */
    public SOSOptionJdbcUrl  gethibernate_connection_url() {
        return hibernate_connection_url;
    }

/**
 * \brief sethibernate_connection_url : 
 * 
 * \details
 * JDBC URL of the database connection
 *
 * @param hibernate_connection_url : 
 */
    public void sethibernate_connection_url (SOSOptionJdbcUrl p_hibernate_connection_url) { 
        this.hibernate_connection_url = p_hibernate_connection_url;
    }

                        

/**
 * \var hibernate_connection_username : 
 * User of the database connection
 *
 */
    @JSOptionDefinition(name = "hibernate_connection_username", 
    description = "", 
    key = "hibernate_connection_username", 
    type = "SOSOptionString", 
    mandatory = false)
    
    public SOSOptionString hibernate_connection_username = new SOSOptionString(this, conClassName + ".hibernate_connection_username", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief gethibernate_connection_username : 
 * 
 * \details
 * User of the database connection
 *
 * \return 
 *
 */
    public SOSOptionString  gethibernate_connection_username() {
        return hibernate_connection_username;
    }

/**
 * \brief sethibernate_connection_username : 
 * 
 * \details
 * User of the database connection
 *
 * @param hibernate_connection_username : 
 */
    public void sethibernate_connection_username (SOSOptionString p_hibernate_connection_username) { 
        this.hibernate_connection_username = p_hibernate_connection_username;
    }

                        

/**
 * \var hibernate_dialect : 
 * Hibernate dialect of the database connection
 *
 */
    @JSOptionDefinition(name = "hibernate_dialect", 
    description = "", 
    key = "hibernate_dialect", 
    type = "SOSOptionString", 
    mandatory = false)
    
    public SOSOptionString hibernate_dialect = new SOSOptionString(this, conClassName + ".hibernate_dialect", // HashMap-Key
                                                                "", // Titel
                                                                "org.hibernate.dialect.Oracle10gDialect", // InitValue
                                                                "org.hibernate.dialect.Oracle10gDialect", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief gethibernate_dialect : 
 * 
 * \details
 * Hibernate dialect of the database connection
 *
 * \return 
 *
 */
    public SOSOptionString  gethibernate_dialect() {
        return hibernate_dialect;
    }

/**
 * \brief sethibernate_dialect : 
 * 
 * \details
 * Hibernate dialect of the database connection
 *
 * @param hibernate_dialect : 
 */
    public void sethibernate_dialect (SOSOptionString p_hibernate_dialect) { 
        this.hibernate_dialect = p_hibernate_dialect;
    }

                        

/**
 * \var hibernate_format_sql : 
 * 
 *
 */
    @JSOptionDefinition(name = "hibernate_format_sql", 
    description = "", 
    key = "hibernate_format_sql", 
    type = "SOSOptionBoolean", 
    mandatory = false)
    
    public SOSOptionBoolean hibernate_format_sql = new SOSOptionBoolean(this, conClassName + ".hibernate_format_sql", // HashMap-Key
                                                                "", // Titel
                                                                "false", // InitValue
                                                                "false", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief gethibernate_format_sql : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionBoolean  gethibernate_format_sql() {
        return hibernate_format_sql;
    }

/**
 * \brief sethibernate_format_sql : 
 * 
 * \details
 * 
 *
 * @param hibernate_format_sql : 
 */
    public void sethibernate_format_sql (SOSOptionBoolean p_hibernate_format_sql) { 
        this.hibernate_format_sql = p_hibernate_format_sql;
    }

                        

/**
 * \var hibernate_show_sql : 
 * 
 *
 */
    @JSOptionDefinition(name = "hibernate_show_sql", 
    description = "", 
    key = "hibernate_show_sql", 
    type = "SOSOptionBoolean", 
    mandatory = false)
    
    public SOSOptionBoolean hibernate_show_sql = new SOSOptionBoolean(this, conClassName + ".hibernate_show_sql", // HashMap-Key
                                                                "", // Titel
                                                                "false", // InitValue
                                                                "false", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief gethibernate_show_sql : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionBoolean  gethibernate_show_sql() {
        return hibernate_show_sql;
    }

/**
 * \brief sethibernate_show_sql : 
 * 
 * \details
 * 
 *
 * @param hibernate_show_sql : 
 */
    public void sethibernate_show_sql (SOSOptionBoolean p_hibernate_show_sql) { 
        this.hibernate_show_sql = p_hibernate_show_sql;
    }

                        

/**
 * \var is_bootstrap_order : 
 * 
 *
 */
    @JSOptionDefinition(name = "is_bootstrap_order", 
    description = "", 
    key = "is_bootstrap_order", 
    type = "SOSOptionBoolean", 
    mandatory = false)
    
    public SOSOptionBoolean is_bootstrap_order = new SOSOptionBoolean(this, conClassName + ".is_bootstrap_order", // HashMap-Key
                                                                "", // Titel
                                                                "false", // InitValue
                                                                "false", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getis_bootstrap_order : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionBoolean  getis_bootstrap_order() {
        return is_bootstrap_order;
    }

/**
 * \brief setis_bootstrap_order : 
 * 
 * \details
 * 
 *
 * @param is_bootstrap_order : 
 */
    public void setis_bootstrap_order (SOSOptionBoolean p_is_bootstrap_order) { 
        this.is_bootstrap_order = p_is_bootstrap_order;
    }

                        

/**
 * \var jobnet : 
 * Name of the job net
 *
 */
    @JSOptionDefinition(name = "jobnet", 
    description = "", 
    key = "jobnet", 
    type = "SOSOptionString", 
    mandatory = false)
    
    public SOSOptionString jobnet = new SOSOptionString(this, conClassName + ".jobnet", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                true // isMandatory
                    );

/**
 * \brief getjobnet : 
 * 
 * \details
 * Name of the job net
 *
 * \return 
 *
 */
    public SOSOptionString  getjobnet() {
        return jobnet;
    }

/**
 * \brief setjobnet : 
 * 
 * \details
 * Name of the job net
 *
 * @param jobnet : 
 */
    public void setjobnet (SOSOptionString p_jobnet) { 
        this.jobnet = p_jobnet;
    }

                        

/**
 * \var max_number_of_checks : 
 * 
 *
 */
    @JSOptionDefinition(name = "max_number_of_checks", 
    description = "", 
    key = "max_number_of_checks", 
    type = "SOSOptionInteger", 
    mandatory = false)
    
    public SOSOptionInteger max_number_of_checks = new SOSOptionInteger(this, conClassName + ".max_number_of_checks", // HashMap-Key
                                                                "", // Titel
                                                                "-1", // InitValue
                                                                "-1", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getmax_number_of_checks : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionInteger  getmax_number_of_checks() {
        return max_number_of_checks;
    }

/**
 * \brief setmax_number_of_checks : 
 * 
 * \details
 * 
 *
 * @param max_number_of_checks : 
 */
    public void setmax_number_of_checks (SOSOptionInteger p_max_number_of_checks) { 
        this.max_number_of_checks = p_max_number_of_checks;
    }

                        

/**
 * \var node : 
 * Name of the job net node
 *
 */
    @JSOptionDefinition(name = "node", 
    description = "", 
    key = "node", 
    type = "SOSOptionString", 
    mandatory = false)
    
    public SOSOptionString node = new SOSOptionString(this, conClassName + ".node", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getnode : 
 * 
 * \details
 * Name of the job net node
 *
 * \return 
 *
 */
    public SOSOptionString  getnode() {
        return node;
    }

/**
 * \brief setnode : 
 * 
 * \details
 * Name of the job net node
 *
 * @param node : 
 */
    public void setnode (SOSOptionString p_node) { 
        this.node = p_node;
    }

                        

/**
 * \var predesessor_nodes : 
 * 
 *
 */
    @JSOptionDefinition(name = "predesessor_nodes", 
    description = "", 
    key = "predesessor_nodes", 
    type = "SOSOptionString", 
    mandatory = false)
    
    public SOSOptionString predesessor_nodes = new SOSOptionString(this, conClassName + ".predesessor_nodes", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getpredesessor_nodes : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionString  getpredesessor_nodes() {
        return predesessor_nodes;
    }

/**
 * \brief setpredesessor_nodes : 
 * 
 * \details
 * 
 *
 * @param predesessor_nodes : 
 */
    public void setpredesessor_nodes (SOSOptionString p_predesessor_nodes) { 
        this.predesessor_nodes = p_predesessor_nodes;
    }

                        

/**
 * \var scheduler_id : 
 * ID of the JobScheduler (Cluster)
 *
 */
    @JSOptionDefinition(name = "scheduler_id", 
    description = "", 
    key = "scheduler_id", 
    type = "SOSOptionSchedulerID", 
    mandatory = false)
    
    public SOSOptionSchedulerID scheduler_id = new SOSOptionSchedulerID(this, conClassName + ".scheduler_id", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getscheduler_id : 
 * 
 * \details
 * ID of the JobScheduler (Cluster)
 *
 * \return 
 *
 */
    public SOSOptionSchedulerID  getscheduler_id() {
        return scheduler_id;
    }

/**
 * \brief setscheduler_id : 
 * 
 * \details
 * ID of the JobScheduler (Cluster)
 *
 * @param scheduler_id : 
 */
    public void setscheduler_id (SOSOptionSchedulerID p_scheduler_id) { 
        this.scheduler_id = p_scheduler_id;
    }

                        

/**
 * \var script_to_execute : 
 * 
 *
 */
    @JSOptionDefinition(name = "script_to_execute", 
    description = "", 
    key = "script_to_execute", 
    type = "SOSOptionScript", 
    mandatory = false)
    
    public SOSOptionScript script_to_execute = new SOSOptionScript(this, conClassName + ".script_to_execute", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getscript_to_execute : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionScript  getscript_to_execute() {
        return script_to_execute;
    }

/**
 * \brief setscript_to_execute : 
 * 
 * \details
 * 
 *
 * @param script_to_execute : 
 */
    public void setscript_to_execute (SOSOptionScript p_script_to_execute) { 
        this.script_to_execute = p_script_to_execute;
    }

                        

/**
 * \var skipDispatcher : 
 * 
 *
 */
    @JSOptionDefinition(name = "skipDispatcher", 
    description = "", 
    key = "skipDispatcher", 
    type = "SOSOptionBoolean", 
    mandatory = false)
    
    public SOSOptionBoolean skipDispatcher = new SOSOptionBoolean(this, conClassName + ".skipDispatcher", // HashMap-Key
                                                                "", // Titel
                                                                "false", // InitValue
                                                                "false", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getskipDispatcher : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionBoolean  getskipDispatcher() {
        return skipDispatcher;
    }

/**
 * \brief setskipDispatcher : 
 * 
 * \details
 * 
 *
 * @param skipDispatcher : 
 */
    public void setskipDispatcher (SOSOptionBoolean p_skipDispatcher) { 
        this.skipDispatcher = p_skipDispatcher;
    }

                        

/**
 * \var skipRunner : 
 * 
 *
 */
    @JSOptionDefinition(name = "skipRunner", 
    description = "", 
    key = "skipRunner", 
    type = "SOSOptionBoolean", 
    mandatory = false)
    
    public SOSOptionBoolean skipRunner = new SOSOptionBoolean(this, conClassName + ".skipRunner", // HashMap-Key
                                                                "", // Titel
                                                                "false", // InitValue
                                                                "false", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getskipRunner : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionBoolean  getskipRunner() {
        return skipRunner;
    }

/**
 * \brief setskipRunner : 
 * 
 * \details
 * 
 *
 * @param skipRunner : 
 */
    public void setskipRunner (SOSOptionBoolean p_skipRunner) { 
        this.skipRunner = p_skipRunner;
    }

                        

/**
 * \var skipWaiter : 
 * 
 *
 */
    @JSOptionDefinition(name = "skipWaiter", 
    description = "", 
    key = "skipWaiter", 
    type = "SOSOptionBoolean", 
    mandatory = false)
    
    public SOSOptionBoolean skipWaiter = new SOSOptionBoolean(this, conClassName + ".skipWaiter", // HashMap-Key
                                                                "", // Titel
                                                                "false", // InitValue
                                                                "false", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getskipWaiter : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionBoolean  getskipWaiter() {
        return skipWaiter;
    }

/**
 * \brief setskipWaiter : 
 * 
 * \details
 * 
 *
 * @param skipWaiter : 
 */
    public void setskipWaiter (SOSOptionBoolean p_skipWaiter) { 
        this.skipWaiter = p_skipWaiter;
    }

                        

/**
 * \var successor_nodes : 
 * 
 *
 */
    @JSOptionDefinition(name = "successor_nodes", 
    description = "", 
    key = "successor_nodes", 
    type = "SOSOptionString", 
    mandatory = false)
    
    public SOSOptionString successor_nodes = new SOSOptionString(this, conClassName + ".successor_nodes", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getsuccessor_nodes : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionString  getsuccessor_nodes() {
        return successor_nodes;
    }

/**
 * \brief setsuccessor_nodes : 
 * 
 * \details
 * 
 *
 * @param successor_nodes : 
 */
    public void setsuccessor_nodes (SOSOptionString p_successor_nodes) { 
        this.successor_nodes = p_successor_nodes;
    }


    @JSOptionDefinition(name = "wait_for_events", 
    description = "", 
    key = "wait_for_events", 
    type = "SOSOptionString", 
    mandatory = false)
    
    public SOSOptionString wait_for_events = new SOSOptionString(this, conClassName + ".wait_for_events", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

    public SOSOptionString  getwait_for_events() {
        return wait_for_events;
    }

    public void setwait_for_events (SOSOptionString events) { 
        this.wait_for_events = events;
    }

/**
 * \var uuid_jobnet_identifier : 
 * 
 *
 */
    @JSOptionDefinition(name = "uuid_jobnet_identifier", 
    description = "", 
    key = "uuid_jobnet_identifier", 
    type = "SOSOptionUUID", 
    mandatory = false)
    
    public SOSOptionUUID uuid_jobnet_identifier = new SOSOptionUUID(this, conClassName + ".uuid_jobnet_identifier", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getuuid_jobnet_identifier : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionUUID  getuuid_jobnet_identifier() {
        return uuid_jobnet_identifier;
    }

/**
 * \brief setuuid_jobnet_identifier : 
 * 
 * \details
 * 
 *
 * @param uuid_jobnet_identifier : 
 */
    public void setuuid_jobnet_identifier (SOSOptionUUID p_uuid_jobnet_identifier) { 
        this.uuid_jobnet_identifier = p_uuid_jobnet_identifier;
    }

                        
    public JobNetOptionsSuperClass() {
		objParentClass = this.getClass();
	} // public JobNetOptionsSuperClass    
        
	//
	public JobNetOptionsSuperClass(HashMap<String, String> JSSettings) throws Exception {
		this();
		this.setAllOptions(JSSettings);
	} // public JobNetOptionsSuperClass (HashMap JSSettings)


	/**
	 * \brief setAllOptions - bernimmt die OptionenWerte aus der HashMap
	 *
	 * \details In der als Parameter anzugebenden HashMap sind Schlssel (Name)
	 * und Wert der jeweiligen Option als Paar angegeben. Ein Beispiel fr den
	 * Aufbau einer solchen HashMap findet sich in der Beschreibung dieser
	 * Klasse (\ref TestData "setJobSchedulerSSHJobOptions"). In dieser Routine
	 * werden die Schlssel analysiert und, falls gefunden, werden die
	 * dazugehrigen Werte den Properties dieser Klasse zugewiesen.
	 *
	 * Nicht bekannte Schlssel werden ignoriert.
	 *
	 * \see JSOptionsClass::getItem
	 *
	 * @param pobjJSSettings
	 * @throws Exception
	 */
	public void setAllOptions(HashMap<String, String> pobjJSSettings) throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setAllOptions";
		flgSetAllOptions = true;
		objSettings = pobjJSSettings;
		super.Settings(objSettings);
		super.setAllOptions(pobjJSSettings);
		flgSetAllOptions = false;
	} // public void setAllOptions (HashMap <String, String> JSSettings)

	/**
	 * \brief CheckMandatory - prft alle Muss-Optionen auf Werte
	 *
	 * \details
	 * @throws Exception
	 *
	 * @throws Exception
	 * - wird ausgelst, wenn eine mandatory-Option keinen Wert hat
	 */
	@Override
	public void CheckMandatory() throws JSExceptionMandatoryOptionMissing //
			, Exception {
		try {
			super.CheckMandatory();
		}
		catch (Exception e) {
			throw new JSExceptionMandatoryOptionMissing(e.toString());
		}
	} // public void CheckMandatory ()

	/**
	 *
	 * \brief CommandLineArgs - bernehmen der Options/Settings aus der
	 * Kommandozeile
	 *
	 * \details Die in der Kommandozeile beim Starten der Applikation
	 * angegebenen Parameter werden hier in die HashMap bertragen und danach
	 * den Optionen als Wert zugewiesen.
	 *
	 * \return void
	 *
	 * @param pstrArgs
	 * @throws Exception
	 */
	@Override
	public void CommandLineArgs(String[] pstrArgs) throws Exception {
		super.CommandLineArgs(pstrArgs);
		this.setAllOptions(super.objSettings);
	}
} // public class JobNetOptionsSuperClass
