/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.plugin;

import com.sos.scheduler.engine.kernel.log.PrefixLog;
import com.sos.scheduler.engine.kernel.scheduler.SchedulerException;
import org.w3c.dom.Element;

import javax.annotation.Nullable;

import static com.sos.scheduler.engine.kernel.plugin.ActivationMode.activateOnStart;
import static com.sos.scheduler.engine.kernel.plugin.ActivationMode.dontActivate;
import static com.sos.scheduler.engine.kernel.util.XmlUtils.booleanXmlAttribute;
import static com.sos.scheduler.engine.kernel.util.XmlUtils.elementXPathOrNull;

public class PluginReader {
    private final PrefixLog log;

    PluginReader(PrefixLog log) {
        this.log = log;
    }

    @Nullable final PluginAdapter tryReadPlugin(Element e) {
        String className = e.getAttribute("java_class");
        if (className.isEmpty()) throw new SchedulerException("Missing attribute java_class in <plugin>");
        return tryReadPlugin(new PluginConfiguration(
                className,
                booleanXmlAttribute(e, "dont_activate", false)? dontActivate : activateOnStart ,
                elementXPathOrNull(e, "plugin.config")));
    }

    @Nullable private PluginAdapter tryReadPlugin(PluginConfiguration c) {
        try {
            return pluginAdapter(c);
        } catch (Exception x) {
            PluginSubsystem.logError(log, "Plugin "+ c.getClassName(), x);
            return null;
        }
    }

    private PluginAdapter pluginAdapter(PluginConfiguration c)  {
        return new PluginAdapter(c, log);
    }
}
