/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.variable;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.sos.scheduler.engine.cplusplus.runtime.Sister;
import com.sos.scheduler.engine.cplusplus.runtime.SisterType;
import com.sos.scheduler.engine.cplusplus.runtime.annotation.ForCpp;
import com.sos.scheduler.engine.kernel.cppproxy.Variable_setC;
import com.sos.scheduler.engine.kernel.scheduler.SchedulerException;

import javax.annotation.Nullable;

@ForCpp public final class VariableSet implements Sister, UnmodifiableVariableSet {
    private final Variable_setC cppProxy;

    VariableSet(Variable_setC cppProxy) {
        this.cppProxy = cppProxy;
    }

    @Override public void onCppProxyInvalidated() {}

    public int size() {
        return cppProxy.count();
    }

    @SuppressWarnings("unchecked")
    public ImmutableList<String> getNames() {
        return ImmutableList.copyOf(cppProxy.java_names());
    }

    @Override @Nullable public String tryGet(String name) {
        return cppProxy.get_string(name);
    }

    @Override public String get(String name) {
        String result = tryGet(name);
        if (result == null)  throw new SchedulerException("Missing parameter '" + name + "'");
        return result;
    }

    public void put(String name, String value) {
        cppProxy.set_var(name, value);
    }

    public ImmutableMap<String,String> toMap() {
        ImmutableMap.Builder<String,String> result = ImmutableMap.builder();
        for (String name: getNames()) result.put(name, get(name));
        return result.build();
    }

//    public final Map<String,String> asMap() {
//        final VariableSet variableSet = this;
//
//        return new Map<String,String>() {
//            @Override public int size() {
//                return variableSet.size();
//            }
//
//            @Override public boolean isEmpty() {
//                return size() > 0;
//            }
//
//            @Override public boolean containsKey(Object key) {
//                return get(key) != null;
//            }
//
//            @Override public String get(Object key) {
//                return key instanceof String? variableSet.tryGet((String)key) : null;
//            }
//
//            @Override public String put(String key, String value) {
//                variableSet.put(key, value);
//            }
//        };
//    }

    public static class Type implements SisterType<VariableSet, Variable_setC> {
        @Override public final VariableSet sister(Variable_setC proxy, Sister context) {
            assert context == null;
            return new VariableSet(proxy);
        }
    }
}
